from typing import Optional

import numpy as np
import pandas as pd


def rolling_mean(
    arr: np.ndarray, window: int, min_count: Optional[int] = None
) -> np.ndarray:
    """Rolling mean of an array.

    Args:
        arr (np.ndarray): Array to calculate rolling mean.
        window (int): Window size.
        min_count (int, optional): Minimum number of non-NaN values in window. Defaults to None.

    Returns:
        np.ndarray: Rolling mean of array.
    """
    return pd.Series(arr).rolling(window, min_periods=min_count).mean().values


def rolling_sum(
    arr: np.ndarray, window: int, min_count: Optional[int] = None
) -> np.ndarray:
    """Rolling sum of an array.

    Args:
        arr (np.ndarray): Array to calculate rolling sum.
        window (int): Window size.
        min_count (int, optional): Minimum number of non-NaN values in window. Defaults to None.

    Returns:
        np.ndarray: Rolling sum of array.
    """
    return pd.Series(arr).rolling(window, min_periods=min_count).sum().values


def rolling_min(
    arr: np.ndarray, window: int, min_count: Optional[int] = None
) -> np.ndarray:
    """Rolling min of an array.

    Args:
        arr (np.ndarray): Array to calculate rolling min.
        window (int): Window size.
        min_count (int, optional): Minimum number of non-NaN values in window. Defaults to None.

    Returns:
        np.ndarray: Rolling min of array.
    """
    return pd.Series(arr).rolling(window, min_periods=min_count).min().values


def rolling_max(
    arr: np.ndarray, window: int, min_count: Optional[int] = None
) -> np.ndarray:
    """Rolling max of an array.

    Args:
        arr (np.ndarray): Array to calculate rolling max.
        window (int): Window size.
        min_count (int, optional): Minimum number of non-NaN values in window. Defaults to None.

    Returns:
        np.ndarray: Rolling max of array.
    """
    return pd.Series(arr).rolling(window, min_periods=min_count).max().values


def rolling_std(
    arr: np.ndarray, window: int, min_count: Optional[int] = None
) -> np.ndarray:
    """Rolling standard deviation of an array.

    Args:
        arr (np.ndarray): Array to calculate rolling standard deviation.
        window (int): Window size.
        min_count (int, optional): Minimum number of non-NaN values in window. Defaults to None.

    Returns:
        np.ndarray: Rolling standard deviation of array.
    """
    return pd.Series(arr).rolling(window, min_periods=min_count).std().values


def rolling_var(
    arr: np.ndarray, window: int, min_count: Optional[int] = None
) -> np.ndarray:
    """Rolling variance of an array.

    Args:
        arr (np.ndarray): Array to calculate rolling variance.
        window (int): Window size.
        min_count (int, optional): Minimum number of non-NaN values in window. Defaults to None.

    Returns:
        np.ndarray: Rolling variance of array.
    """
    return pd.Series(arr).rolling(window, min_periods=min_count).var().values


def rolling_skew(
    arr: np.ndarray, window: int, min_count: Optional[int] = None
) -> np.ndarray:
    """Rolling skewness of an array.

    Args:
        arr (np.ndarray): Array to calculate rolling skewness.
        window (int): Window size.
        min_count (int, optional): Minimum number of non-NaN values in window. Defaults to None.

    Returns:
        np.ndarray: Rolling skewness of array.
    """
    return pd.Series(arr).rolling(window, min_periods=min_count).skew().values


def rolling_kurt(
    arr: np.ndarray, window: int, min_count: Optional[int] = None
) -> np.ndarray:
    """Rolling kurtosis of an array.

    Args:
        arr (np.ndarray): Array to calculate rolling kurtosis.
        window (int): Window size.
        min_count (int, optional): Minimum number of non-NaN values in window. Defaults to None.

    Returns:
        np.ndarray: Rolling kurtosis of array.
    """
    return pd.Series(arr).rolling(window, min_periods=min_count).kurt().values



import numpy as np
from numpy.lib.stride_tricks import sliding_window_view


def pad(
    arr: np.ndarray, n: int, axis: int = -1, constant_values: float = np.nan
) -> np.ndarray:
    widths = [(0, 0) for _ in range(arr.ndim)]
    widths[axis] = (n, 0) if n > 0 else (0, -n)
    return np.pad(arr, widths, "constant", constant_values=constant_values)


def sliding_window(arr: np.ndarray, window: int, axis: int = -1) -> np.ndarray:
    return sliding_window_view(arr, window, axis=axis)


def rolling_sum(arr: np.ndarray, window: int, axis: int = -1) -> np.ndarray:
    return pad(
        sliding_window(arr, window, axis=axis).sum(axis=-1).astype(np.float64),
        window - 1,
        axis,
    )


def rolling_mean(arr: np.ndarray, window: int, axis: int = -1) -> np.ndarray:
    return pad(
        sliding_window(arr, window, axis=axis).mean(axis=-1).astype(np.float64),
        window - 1,
        axis,
    )


def rolling_std(arr: np.ndarray, window: int, axis: int = -1) -> np.ndarray:
    return pad(
        sliding_window(arr, window, axis=axis).std(axis=-1).astype(np.float64),
        window - 1,
        axis,
    )


def rolling_max(arr: np.ndarray, window: int, axis: int = -1) -> np.ndarray:
    return pad(
        sliding_window(arr, window, axis=axis).max(axis=-1).astype(np.float64),
        window - 1,
        axis,
    )


def rolling_min(arr: np.ndarray, window: int, axis: int = -1) -> np.ndarray:
    return pad(
        sliding_window(arr, window, axis=axis).min(axis=-1).astype(np.float64),
        window - 1,
        axis,
    )



__version__ = "0.3.7"

from .rolling import (  # rolling_kurt,; rolling_skew,; rolling_std,; rolling_var,
    rolling_argmax,
    rolling_argmin,
    rolling_max,
    rolling_mean,
    rolling_median,
    rolling_min,
    rolling_quantile,
    rolling_rank,
    rolling_sum,
)



from pathlib import Path

import numpy as np
from setuptools import Extension, find_packages, setup

project_dir = Path(__file__).parent.resolve()
long_description = (project_dir / "README.md").read_text(encoding="utf-8")

setup(
    name="cougar",
    version="0.3.7",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Yunchong Gan",
    author_email="yunchong@pku.edu.cn",
    packages=find_packages(),
    requires=["numpy"],
    install_requires=["numpy"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "Natural Language :: English",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: C",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    ext_modules=[
        Extension(
            "cougar.rolling",
            ["cougar/rolling.c"],
            include_dirs=["cougar", np.get_include()],
            extra_compile_args=["-O2"],
        )
    ],
)
